<?php

namespace Kolab\FreeBusy;

/**
 * Abstract class representing an address directory for free/busy data lookups
 */
abstract class Directory
{
	protected $config;

	/**
	 * Factory method creating an instace of Directory according to config
	 *
	 * @param array Hash array with config
	 */
	public static function factory($config)
	{
		switch (strtolower($config['type'])) {
			case 'ldap':
				return new DirectoryLDAP($config);

			case 'static':
			case 'external':
				return new DirectoryStatic($config);

			default:
				Logger::get('directory')->addError("Invalid directory type '" . $config['type'] . "'!");
		}

		return null;
	}

	/**
	 * Resolve the given username to a Entity object
	 *
	 * @param string Username/Email to resolve
	 * @return object Entity if found, otherwise False
	 */
	abstract public function resolve($user);

	/**
	 * Retrieve free/busy data for the given user.
	 *
	 * @param string Username or email to resolve
	 * @param boolean Get extemded free-busy if possible
	 * @return string VCalendar container if found, False otherwise
	 */
	public function getFreeBusyData($user, $extended = false)
	{
		// resolve user record first
		if ($user = $this->resolve($user)) {
			$fbsource = $this->config['fbsource'];
			if ($source = Source::Factory($fbsource)) {
				// forward request to Source instance
				if ($data = $source->getFreeBusyData($user, $extended)) {
					// send data through the according format converter
					$converter = Format::factory($this->config['format']);
					$data = $converter->toVCalendar($data);
				}

				return $data;
			}
		}

		return false;
	}

}